﻿//
//Keyboard Filter Utility
//Annabooks, LLC.
//Copyright (c) 2016 Annabooks, LLC. All Rights Reserved
//
//
//The sample application demonstrates some basic Keyboard Filter WMI APIs in C# and controlling the Keyboard Filter Service
//
//
//Date: 12/2/2016 - Comments generated
//
//Source code provided as is without any warranty
//


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.ServiceProcess;
using System.Runtime.InteropServices;
using System.Security;

namespace KBFUtility
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            CheckKbFStart();
            kbStatus();
        }

        //Start the keyboard service
        private void btnEnable_Click(object sender, EventArgs e)
        {
            ServiceController sc = new ServiceController("MsKeyboardFilter");

            try
            {
                sc.Start();
                sc.WaitForStatus(ServiceControllerStatus.Running, new TimeSpan(0, 0, 0, 20));
                kbStatus();
            }
            catch (Exception ex)
            {
                lblStatus.Text = "Error: " + ex;
            }

        }

        //Stop the keyboard service
        private void button2_Click(object sender, EventArgs e)
        {
            ServiceController sc = new ServiceController("MsKeyboardFilter");

            try
            {
                sc.Stop();
                sc.WaitForStatus(ServiceControllerStatus.Stopped, new TimeSpan(0, 0, 0, 20));
                kbStatus();
            }
            catch (Exception ex)
            {
                lblStatus.Text = "Error: " + ex;
            }


        }

        //Return the service status
        //Get serice status code: http://stackoverflow.com/questions/178147/how-can-i-verify-if-a-windows-service-is-running
        public void kbStatus()
        {
            ServiceController sc = new ServiceController("MsKeyboardFilter");

            switch (sc.Status)
            {
                case ServiceControllerStatus.Running:
                    txtStatus.Text = "Running";
                    break;
                case ServiceControllerStatus.Stopped:
                    txtStatus.Text = "Stopped";
                    break;
                case ServiceControllerStatus.Paused:
                    txtStatus.Text = "Paused";
                    break;
                case ServiceControllerStatus.StopPending:
                    txtStatus.Text = "Stopping";
                    break;
                case ServiceControllerStatus.StartPending:
                    txtStatus.Text = "Starting";
                    break;
                default:
                    txtStatus.Text = "Status Changing";
                    break;
            }

        }

        //Check to see if Keyboard Filter Service is disabled. If the KBF is disabled, set the start mode to manual
        public void CheckKbFStart()
        {
            ServiceController sc = new ServiceController("MsKeyboardFilter");

            if (sc.StartType.ToString() == "Disabled")
            {
                ServiceHelper.ChangeStartMode(sc, ServiceStartMode.Manual);
            }
        }

    }

    //Nice class to help with chaning start type - P/invoke
    //Change start mode discussion: http://stackoverflow.com/questions/1477618/how-do-i-change-a-windows-services-startup-type-in-net-post-install
    //Change start mode solution from http://peterkellyonline.blogspot.com/2011/04/configuring-windows-service.html

    public static class ServiceHelper
    {
        [DllImport("advapi32.dll", CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern Boolean ChangeServiceConfig(
            IntPtr hService,
            UInt32 nServiceType,
            UInt32 nStartType,
            UInt32 nErrorControl,
            String lpBinaryPathName,
            String lpLoadOrderGroup,
            IntPtr lpdwTagId,
            [In] char[] lpDependencies,
            String lpServiceStartName,
            String lpPassword,
            String lpDisplayName);

        [DllImport("advapi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern IntPtr OpenService(
            IntPtr hSCManager, string lpServiceName, uint dwDesiredAccess);

        [DllImport("advapi32.dll", EntryPoint = "OpenSCManagerW", ExactSpelling = true, CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern IntPtr OpenSCManager(
            string machineName, string databaseName, uint dwAccess);

        [DllImport("advapi32.dll", EntryPoint = "CloseServiceHandle")]
        public static extern int CloseServiceHandle(IntPtr hSCObject);

        private const uint SERVICE_NO_CHANGE = 0xFFFFFFFF;
        private const uint SERVICE_QUERY_CONFIG = 0x00000001;
        private const uint SERVICE_CHANGE_CONFIG = 0x00000002;
        private const uint SC_MANAGER_ALL_ACCESS = 0x000F003F;

 
        public static void ChangeStartMode(ServiceController svc, ServiceStartMode mode)
        {

            var scManagerHandle = OpenSCManager(null, null, SC_MANAGER_ALL_ACCESS);
            if (scManagerHandle == IntPtr.Zero)
            {
                throw new ExternalException("Open Service Manager Error");
            }

            var serviceHandle = OpenService(
                scManagerHandle,
                svc.ServiceName,
                SERVICE_QUERY_CONFIG | SERVICE_CHANGE_CONFIG);

            if (serviceHandle == IntPtr.Zero)
            {
                throw new ExternalException("Open Service Error");
            }

            var result = ChangeServiceConfig(
                serviceHandle,
                SERVICE_NO_CHANGE,
                (uint)mode,
                SERVICE_NO_CHANGE,
                null,
                null,
                IntPtr.Zero,
                null,
                null,
                null,
                null);

            if (result == false)
            {
                int nError = Marshal.GetLastWin32Error();
                var win32Exception = new Win32Exception(nError);
                throw new ExternalException("Could not change service start type: "
                    + win32Exception.Message);
            }

            CloseServiceHandle(serviceHandle);
            CloseServiceHandle(scManagerHandle);
        }
    }
}

